
const state = {
    customers: [],
    filteredCustomers: [],
    loadingCustomers: false,
    showingCustomerForm: false,
    selectedCustomer: null
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    customers(state) {
        return state.customers
    },
    /**
     * 
     * @param {*} state 
     */
    visibleCustomers(state) {
        return state.filteredCustomers.length === 0 ? state.customers : state.filteredCustomers
    },
    /**
     * 
     * @param {*} state 
     */
    loadingCustomers(state) {
        return state.loadingCustomers
    },
    /**
     * 
     * @param {*} state 
     */
    showingCustomerForm(state) {
        return state.showingCustomerForm
    },
    /**
     * 
     * @param {*} state 
     */
    selectedCustomer(state) {
        return state.selectedCustomer
    },
    /**
     * 
     * @param {*} state 
     */
    selectedCustomerName(state) {
        return state.selectedCustomer && state.selectedCustomer.name
    },
    /**
     * 
     * @param {*} state 
     */
    customerCredit(state) {
        return state.selectedCustomer && state.selectedCustomer.creditLimit >= state.selectedCustomer.credit ? state.selectedCustomer.creditLimit - state.credit : 0
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setCustomers(state, payload) {
        state.customers = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setFilteredCustomers(state, payload) {
        state.filteredCustomers = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingCustomers(state, payload) {
        state.loadingCustomers = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setShowCustomerForm(state, payload) {
        state.showingCustomerForm = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    addCustomer(state, payload) {
        state.customers = [payload, ...state.customers]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSelectedCustomer(state, payload) {
        state.selectedCustomer = payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initCustomers({ commit }, payload) {
        commit('setCustomers', payload)
        commit('setLoadingCustomers')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    filterCustomers({ commit }, payload) {
        commit('setFilteredCustomers', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    showCustomerForm({ commit }) {
        commit('setShowCustomerForm', true)
    },
    /**
     * 
     * @param {*} param0 
     */
    hideCustomerForm({ commit }) {
        commit('setShowCustomerForm', false)
    },
    /**
     * 
     * @param {*} param0 
     */
    submitCustomer({ commit, dispatch }, payload) {
        commit('setLoadingCustomers', true)
        dispatch('createCustomer', payload)
        dispatch('hideCustomerForm')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    receiveCustomer({ commit }, payload) {
        commit('addCustomer', payload)
        commit('setLoadingCustomers', false)
    },
    /**
     * 
     * @param {*} param0 
     */
    selectCustomer({ commit }, payload) {
        commit('setSelectedCustomer', payload)
    },
    /**
     * 
     * @param {*} param0 
     */
    resetCustomer({ commit }) {
        commit('setLoadingCustomers', true)
        commit('setCustomers', [])
        commit('setFilteredCustomers', [])
        commit('setSelectedCustomer', null)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}