const state = {
    products: [],
    filteredProducts: [],
    loadingProducts: false,
    showingProductForm: false,
    showVariants: false,
    productWithVariant: null
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    products(state) {
        return state.products
    },
    /**
     * 
     * @param {*} state 
     */
    visibleProducts(state) {
        return state.filteredProducts.length === 0 ? state.products : state.filteredProducts
    },
    /**
     * 
     * @param {*} state 
     */
    loadingProducts(state) {
        return state.loadingProducts
    },
    /**
     * 
     * @param {*} state 
     */
    showingProductForm(state) {
        return state.showingProductForm
    },
    /**
     * 
     * @param {*} state 
     */
    showVariants(state) {
        return state.showVariants
    },
    /**
     * 
     * @param {*} state 
     */
    productWithVariant(state) {
        return state.productWithVariant
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setProducts(state, payload) {
        state.products = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setFilteredProducts(state, payload) {
        state.filteredProducts = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingProducts(state, payload) {
        state.loadingProducts = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setShowProductForm(state, payload) {
        state.showingProductForm = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    addProduct(state, payload) {
        state.products = [payload, ...state.products]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setProductWithVariant(state, payload) {
        state.productWithVariant = payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initProducts({ commit }, payload) {
        commit('setProducts', payload)
        commit('setLoadingProducts')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    filterProducts({ commit }, payload) {
        commit('setFilteredProducts', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    showProductForm({ commit }) {
        commit('setShowProductForm', true)
    },
    /**
     * 
     * @param {*} param0 
     */
    hideProductForm({ commit }) {
        commit('setShowProductForm', false)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    submitProduct({ commit, dispatch }, payload) {
        commit('setLoadingProducts', true)
        dispatch('createProduct', payload)
        dispatch('hideProductForm')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    receiveProduct({ commit }, payload) {
        commit('addProduct', payload)
        commit('setLoadingProducts', false)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    selectProduct({ commit, dispatch }, payload) {
        if (!payload) {
            commit('setProductWithVariant', null)
            return
        }

        if (payload.variantCount > 1) {
            commit('setProductWithVariant', payload)
            return
        }

        if (!payload.variantCount) {
            dispatch('addToCart', payload)
            commit('setProductWithVariant', null)
        } else {
            dispatch('addToCart', payload.variants[0])
        }
    },
    /**
     * 
     * @param {*} param0 
     */
    resetProduct({ commit }) {
        commit('setLoadingProducts', true)
        commit('setProducts', [])
        commit('setFilteredProducts', [])
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}

